# -*- coding: utf-8 -*-
# -*- Python Version: 3.7 -*-

"""Functions to output a PHX Domain model to  a WUFI XML text document.

The PHX objects here are passed along to the xml_schemas for writing to file, 
and these xml-schemas contain the actual WUFI XML node names and ordering.
"""
