# -*- coding: utf-8 -*-
# -*- Python Version: 3.7 -*-

"""PHX Domain-Model Classes

Project
    ├---Utilization Patterns (Ventilation)
    ├---Utilization Patterns (Passive House)
    ├---Assembly Types
    ├---Window Types
    └---Variants (Building-Segment)
        ├---Climate
        ├---Graphics3D
        └---Building
            ├---Components
            ├---Shades (Components)
            └---Zones
                └---Rooms
"""
